package cn.com.qmth.examcloud.ws.starter.core;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.StringUtil;
import com.google.common.collect.Maps;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.websocket.Session;
import java.util.Date;
import java.util.Map;

/**
 * WebSocket helper
 *
 * @author WANGWEI
 * @date 2019年11月22日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class WebSocketHelper {

    private static final Logger LOG = LoggerFactory.getLogger(WebSocketHelper.class);

    private static final Map<String, Session> INDEX_MAP = Maps.newConcurrentMap();

    private static final Map<Session, SessionInfo> SESESION_INFO_MAP = Maps.newConcurrentMap();

    /**
     * 重新设置session
     *
     * @param path
     * @param user
     * @param session
     * @param sessionId
     * @author WANGWEI
     */
    public static void setSession(String path, User user, Session session, String sessionId) {

        String index = user.getKey() + ":" + path;
        Session oldSession = INDEX_MAP.get(index);
        if (null != oldSession && !oldSession.equals(session)) {
            SESESION_INFO_MAP.remove(oldSession);
            IOUtils.closeQuietly(oldSession);
        }

        INDEX_MAP.put(index, session);

        SessionInfo si = new SessionInfo();
        si.setIndex(index);
        si.setUser(user);
        si.setSessionId(sessionId);
        SESESION_INFO_MAP.put(session, si);

        LOG.info("WS_SESSION_IN key={} cacheSize={}", index, INDEX_MAP.size());
    }

    /**
     * 关闭 session
     *
     * @param session
     * @author WANGWEI
     */
    public static void closeSession(Session session) {
        SessionInfo si = SESESION_INFO_MAP.get(session);

        if (null != si) {
            INDEX_MAP.remove(si.getIndex());

            LOG.info("WS_SESSION_OUT key={} cacheSize={}", si.getIndex(), INDEX_MAP.size());
        }

        SESESION_INFO_MAP.remove(session);

        IOUtils.closeQuietly(session);
    }

    /**
     * 获取 session
     *
     * @param path
     * @param key
     * @return
     * @author WANGWEI
     */
    public static Session getSession(String path, String key) {
        String index = key + ":" + path;
        Session session = INDEX_MAP.get(index);
        return session;
    }

    /**
     * 获取 session
     *
     * @param path
     * @param rootOrgId
     * @param userType
     * @param userId
     * @return
     * @author WANGWEI
     */
    public static Session getSession(String path, Long rootOrgId, UserType userType, Long userId) {

        String key = StringUtil.join("U_", userType.getCode(), "_", rootOrgId, "_", userId);

        return getSession(path, key);
    }

    /**
     * 获取 SessionInfo
     *
     * @param session
     * @return
     * @author WANGWEI
     */
    public static SessionInfo getSessionInfo(Session session) {
        return SESESION_INFO_MAP.get(session);
    }

    /**
     * 发送消息
     *
     * @param session
     * @param path
     * @param out
     * @author WANGWEI
     */
    public static void sendText(Session session, String path, MessageOut out) {
        try {
            out.setDate(new Date());
            String message = JsonUtil.toJson(out);
            if (LOG.isDebugEnabled()) {
                LOG.debug("[sendText]. path=" + path + "; message=" + message);
            }
            session.getBasicRemote().sendText(message);
        } catch (Exception e) {
            LOG.error("[sendText-FAIL]. path=" + path, e);
            IOUtils.closeQuietly(session);
            return;
        }
    }

    /**
     * 发送消息
     *
     * @param session
     * @param path
     * @param out
     * @author WANGWEI
     */
    public static void sendText(Session session, String path, String out) {
        try {
            if (LOG.isDebugEnabled()) {
                LOG.error("[sendText]. path=" + path + "; message=" + out);
            }
            session.getBasicRemote().sendText(out);
        } catch (Exception e) {
            LOG.error("[sendText-FAIL]. path=" + path, e);
            IOUtils.closeQuietly(session);
            return;
        }
    }

}
